﻿#region Usings

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Xml.Linq;
using FeedReader.Data;
using FeedReader.Expections;
using FeedReader.Xml;
using Image = System.Drawing.Image;
using FeedReader.Xml.Interfaces;

#endregion

namespace FeedReader
{
    public class Feed : IDisposable
    {
        private static bool _useCache;
        private bool _isDisposed;
        private static List<FeedItemFilter> _feedItemFilters = new List<FeedItemFilter>();

        public delegate void OnNewItemsEventHandler(Feed feed, List<FeedItem> newItems);
        public event OnNewItemsEventHandler OnNewItems;

        public Feed(string url_path, string cacheFolder, List<FeedItemFilter> itemFilters)
        {
            LogEvents.InvokeOnDebug(new FeedArgs("Add new feed " + url_path + " using cache folder " + cacheFolder));
            UrlPath = url_path;
            if (!string.IsNullOrEmpty(cacheFolder))
            {
                if (Utils.IsValidPath(cacheFolder))
                {
                    LogEvents.InvokeOnDebug(new FeedArgs(cacheFolder + " is a valid path. Now checking if the cache folder already exists"));
                    if (!Directory.Exists(cacheFolder))
                    {
                        LogEvents.InvokeOnDebug(new FeedArgs(cacheFolder + " doesn't exist. Now create a new folder."));
                        Directory.CreateDirectory(cacheFolder);
                    }
                    LogEvents.InvokeOnDebug(new FeedArgs(cacheFolder + " exist. Caching for feed is now enabled."));
                    if (!cacheFolder.EndsWith(@"\")) cacheFolder += @"\";
                    CacheFolder = cacheFolder;
                    _useCache = true;
                    _feedItemFilters = itemFilters;
                }
                else
                {
                    _useCache = false;
                    throw new FeedCacheFolderNotValid(cacheFolder + " is not a valid path. Caching disabled");
                }
            }
            else
            {
                _useCache = false;
                throw new FeedNoCacheFolderExpection("Cache folder path is empty. Caching disabled");
            }
            LastUpdateSuccessful = false;
            Items = new List<FeedItem>();
            //Update();
        }

        public Feed(string url_path)
        {
            LogEvents.InvokeOnDebug(new FeedArgs("Add new feed without using cache"));
            UrlPath = url_path;
            _useCache = false;
            LastUpdateSuccessful = false;
            Items = new List<FeedItem>();
            //Update();
        }

        public Feed()
        {
            Items = new List<FeedItem>();
        }

        public static bool DeleteCache()
        {
            LogEvents.InvokeOnDebug(new FeedArgs("Try to delete cache"));
            if (Directory.Exists(CacheFolder))
            {
                try
                {
                    Directory.Delete(CacheFolder, true);
                    LogEvents.InvokeOnInfo(new FeedArgs("Deleted cache successful"));
                    return true;
                }
                catch (Exception ex)
                {
                    LogEvents.InvokeOnError(new FeedArgs("Error deleting cache", ex.Message, ex.StackTrace));
                }

            }
            LogEvents.InvokeOnInfo(new FeedArgs("Error deleting cache. Directory " + CacheFolder + " doesn't exist"));
            return false;
        }

        public string Title { get; set; }
        public string Description { get; set; }
        public static string CacheFolder { get; protected set; }
        public List<FeedItem> Items { get; set; }
        public FeedType Type { get; set; }
        public Image Image { get; set; }
        public string ImagePath { get; set; }
        public string UrlPath { get; set; }
        public DateTime LastUpdate { get; set; }
        public bool LastUpdateSuccessful { get; set; }

        public virtual bool Update(bool downloadImages)
        {
            LastUpdateSuccessful = false;

            LogEvents.InvokeOnDebug(new FeedArgs("Try downloading/loading feed from " + UrlPath));
            XDocument xmlFeed;
            try
            {
                xmlFeed = XDocument.Load(UrlPath);
                LogEvents.InvokeOnDebug(new FeedArgs("Download/Loading of feed " + UrlPath + " successful"));
            }
            catch (Exception ex)
            {
                LogEvents.InvokeOnError(new FeedArgs("Error downloading/loading feed from url/path " + UrlPath, ex.Message, ex.StackTrace));
                return false;
            }

            List<FeedItem> oldItems = new List<FeedItem>();
            bool parseSuccess = false;
            List<IFeedTypeXmlParser> feedParsers = new List<IFeedTypeXmlParser>();
            feedParsers.Add(new FeedRdfXmlParser());
            feedParsers.Add(new FeedRssXmlParser());
            feedParsers.Add(new FeedAtomXmlParser());
            
            foreach (IFeedTypeXmlParser feedParser in feedParsers)
            {
                if (!_useCache)
                {
                    LogEvents.InvokeOnInfo(new FeedArgs("Parsing feed from url/path " + UrlPath + " without using cache"));
                    parseSuccess = feedParser.TryParse(xmlFeed, downloadImages, _feedItemFilters);
                }
                else
                {
                    LogEvents.InvokeOnInfo(new FeedArgs("Parsing feed from url/path " + UrlPath + " into cache folder " + CacheFolder, ""));
                    parseSuccess = feedParser.TryParse(xmlFeed, CacheFolder, downloadImages, _feedItemFilters);
                }
                if (parseSuccess)
                {
                    Description = feedParser.GetParsedFeed().Description;
                    LastUpdate = DateTime.Now;
                    LastUpdateSuccessful = true;
                    Image = feedParser.GetParsedFeed().Image;
                    oldItems = Items.CloneList<FeedItem>();
                    Items = feedParser.GetParsedFeed().Items;
                    Title = feedParser.GetParsedFeed().Title;
                    ImagePath = feedParser.GetParsedFeed().ImagePath;
                    if (feedParser is FeedRdfXmlParser) Type = FeedType.RDF;
                    if (feedParser is FeedRssXmlParser) Type = FeedType.RSS;
                    if (feedParser is FeedAtomXmlParser) Type = FeedType.ATOM;
                    LogEvents.InvokeOnInfo(new FeedArgs("Parsing " + Type.ToString() + " feed from url/path " + UrlPath + " successfull"));
                    break;
                }
            }

            if (!parseSuccess)
            {
                LastUpdateSuccessful = false;
                LogEvents.InvokeOnInfo(new FeedArgs("Parsing feed from url/path " + UrlPath + " unsuccessful. See above for errors or warnings"));
                return false;
            }
            else
            {
                CheckForNewItems(oldItems, Items);
                return true;
            }
        }

        private void CheckForNewItems(List<FeedItem> oldItems, List<FeedItem> items)
        {
            if (oldItems.Count >= 1 && items.Count >= 1 && OnNewItems != null)
            {
                List<FeedItem> newItems = items.Except(oldItems, new FeedItemComparer()).ToList();
                if (newItems.Count > 0)
                {
                    OnNewItems((Feed) this.Clone(), newItems.CloneList<FeedItem>());
                }
            }
        }

        public virtual object Clone()
        {
            Feed feed = new Feed();
            feed.Title = this.Title;
            feed.Description = this.Description;
            feed.Items = this.Items.CloneList<FeedItem>();
            feed.Type = this.Type;
            feed.Image = this.Image;
            feed.ImagePath = this.ImagePath;
            feed.UrlPath = this.UrlPath;
            feed.LastUpdate = this.LastUpdate;
            feed.LastUpdateSuccessful = this.LastUpdateSuccessful;
            return feed;
        }

        #region IDisposable Members

        /// <summary>
        /// Performs the disposal.
        /// </summary>
        private void Dispose(bool disposing)
        {
            if (disposing && !_isDisposed)
            {
                this.Title = null;
                this.Description = null;
                this.Items.Clear();
                this.Items = null;
                this.Image.Dispose();
                this.Image = null;
                this.ImagePath = null;
                this.UrlPath = null;
            }

            _isDisposed = true;
        }

        /// <summary>
        /// Releases the object to the garbage collector
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion


    }
}
